"""
Version information for NetworkX, created during installation.

Do not add this file to the repository.

"""

import datetime

version = '1.6rc1'
date = 'Tue Nov 15 14:56:41 2011'

# Was NetworkX built from a development version? If so, remember that the major
# and minor versions reference the "target" (rather than "current") release.
dev = False

# Format: (name, major, min, revision)
version_info = ('networkx', 1, '6rc1', '44274d7a86b8')

# Format: a 'datetime.datetime' instance
date_info = datetime.datetime(2011, 11, 15, 14, 56, 41, 223534)

# Format: (vcs, vcs_tuple)
vcs_info = ('mercurial', ('44274d7a86b8', 'tip'))

